import "Turbine.Gameplay";


--[[
	-- define Resources
--]]

local defaultResources = {
	  Capacity = {
				  Base = "PTweety/BigPack/Resources/storage_fill_base.tga"
				, Frame = "PTweety/BigPack/Resources/storage_fill_frame.tga"
			}
	, Search = {
				  Background = "PTweety/BigPack/Resources/searchbar_background.tga"
			   }
	, Item = {
				  Background = "PTweety/BigPack/Resources/item_empty_small.tga"
			  }
};


--[[
	-- define BigPackWindow
--]]

BigPackWindow = class( Turbine.UI.Lotro.Window );

function BigPackWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );
	
	self.Defaults = { };
	self.Settings = { };

	self:SetDefaults();
	self:Load();
	
	self:SetPosition( self.Settings.BPW_Left, self.Settings.BPW_Top );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetSize( self.Settings.BPW_Width, self.Settings.BPW_Height );
	self:SetText( self.Settings.BPW_WindowTitle );
	self:SetVisible( true );

	self:SetOpacity( self.Settings.BPW_Opacity );

	self.MouseEnter = function( sender, args )
		sender:SetOpacity( 1 );
	end

	self.MouseLeave = function( sender, args )
		sender:SetOpacity( self.Settings.BPW_Opacity );
	end

	self.PositionChanged = function( sender, args )
		self.Settings.BPW_Left		= self:GetLeft();
		self.Settings.BPW_Top		= self:GetTop();
		self:Save();
	end

	self.SizeChanged = function( sender, args )
		local width, height = self:GetSize();
		self.Settings.BPW_Width		= width;
		self.Settings.BPW_Height	= height;
		self:Save();
	end

	-- bottomP(anel) and its controls
	self.bottonP = { };
	self.bottonP.PosOffsetX = 0;
	self.bottonP.PosOffsetY = 0;
	self.bottonP.SizeY = 25;

	-- topP(anel) and its controls
	self.topP = Turbine.UI.Control();
	self.topP:SetParent( self );
	self.topP.PosOffsetX = 5;
	self.topP.PosOffsetY = 37;
	self.topP.SizeY = 48 + 22 + 6;
	self.topP:SetPosition( self.topP.PosOffsetX, self.topP.PosOffsetY );
	self.topP:SetSize( self:GetWidth() - self.topP.PosOffsetX * 2, self.topP.SizeY );

	self.topP.capP = Turbine.UI.Control();
	self.topP.capP:SetParent( self.topP );
	self.topP.capP:SetSize( 381, 48 );
	self.topP.capP:SetPosition( ( self.topP:GetWidth() - self.topP.capP:GetWidth() ) / 2, 22 );

	self.topP.capP.FillBase = Turbine.UI.Control();
	self.topP.capP.FillBase:SetParent( self.topP.capP );
	self.topP.capP.FillBase:SetSize( 376, 24 );
	self.topP.capP.FillBase:SetPosition( ( self.topP.capP:GetWidth() - self.topP.capP.FillBase:GetWidth() ) / 2, 11 );
	self.topP.capP.FillBase:SetBackground( defaultResources.Capacity.Base );

	self.topP.capP.Filler = Turbine.UI.Control();
	self.topP.capP.Filler:SetParent( self.topP.capP );
	self.topP.capP.Filler:SetPosition( 0, 14 );
	self.topP.capP.Filler.MaxWidth = self.topP.capP:GetWidth() - self.topP.capP.Filler:GetLeft() * 2;
	self.topP.capP.Filler:SetSize( self.topP.capP.Filler.MaxWidth, 15 );
	self.topP.capP.Filler:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay );

	self.topP.capP.Empty = Turbine.UI.Label();
	self.topP.capP.Empty:SetParent( self.topP.capP );
	self.topP.capP.Empty:SetPosition( 11, 14 );
	self.topP.capP.Empty:SetSize( 44, 20 );
	self.topP.capP.Empty:SetTextAlignment( Turbine.UI.ContentAlignment.LeftCenter );
	self.topP.capP.Empty:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 );
	self.topP.capP.Empty:SetForeColor( Turbine.UI.Color(0,.7,0) );
	self.topP.capP.Empty:SetOutlineColor( Turbine.UI.Color(0,0,0) );
	self.topP.capP.Empty:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.topP.capP.Empty:SetText( L'empty' );

	self.topP.capP.Display = Turbine.UI.Label();
	self.topP.capP.Display:SetParent( self.topP.capP );
	self.topP.capP.Display:SetPosition( 0, 12 );
	self.topP.capP.Display:SetSize( self.topP.capP:GetWidth(), 20 );
	self.topP.capP.Display:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.topP.capP.Display:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 );
	self.topP.capP.Display:SetForeColor( Turbine.UI.Color(1,1,1) );
	self.topP.capP.Display:SetOutlineColor( Turbine.UI.Color(0,0,0) );
	self.topP.capP.Display:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.topP.capP.Display:SetText( L'capacity' );

	self.topP.capP.Full = Turbine.UI.Label();
	self.topP.capP.Full:SetParent( self.topP.capP );
	self.topP.capP.Full:SetSize( 44, 20 );
	self.topP.capP.Full:SetPosition( self.topP.capP:GetWidth() - self.topP.capP.Full:GetWidth(), 14 );
	self.topP.capP.Full:SetTextAlignment( Turbine.UI.ContentAlignment.RightCenter );
	self.topP.capP.Full:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 );
	self.topP.capP.Full:SetForeColor( Turbine.UI.Color(.75,0,0) );
	self.topP.capP.Full:SetOutlineColor( Turbine.UI.Color(0,0,0) );
	self.topP.capP.Full:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.topP.capP.Full:SetText( L'full' );

	self.topP.capP.FillFrame = Turbine.UI.Control();
	self.topP.capP.FillFrame:SetParent( self.topP.capP );
	self.topP.capP.FillFrame:SetSize( 381, 48 );
	self.topP.capP.FillFrame:SetPosition( (self.topP.capP:GetWidth() - self.topP.capP.FillFrame:GetWidth() ) / 2, 0 );
	self.topP.capP.FillFrame:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	self.topP.capP.FillFrame:SetBackground( defaultResources.Capacity.Frame );
	
	self.topP.SizeChanged = function( sender, args )
		local width, height = self.topP:GetSize();
		self.topP.capP:SetLeft( ( width - self.topP.capP:GetWidth() ) / 2 );
	end

	-- searchP(anel) and its controls
	self.searchP = Turbine.UI.Control();
	self.searchP:SetParent( self );
	self.searchP.PosOffsetX = 7;
	self.searchP.PosOffsetY = 2;
	self.searchP.SizeY = 32;
	self.searchP:SetSize( self:GetWidth() - self.searchP.PosOffsetX * 2, self.searchP.SizeY );
	self.searchP:SetPosition( self.searchP.PosOffsetX, self.topP.PosOffsetY + self.topP:GetHeight() + self.searchP.PosOffsetY );
	self.searchP:SetBackground( defaultResources.Search.Background );

	self.searchP.SearchLabel = Turbine.UI.Label();
	self.searchP.SearchLabel:SetParent( self.searchP );
	self.searchP.SearchLabel:SetPosition( 5, 6 );
	self.searchP.SearchLabel:SetSize( 50, 20 );
	self.searchP.SearchLabel:SetOutlineColor( Turbine.UI.Color(0,0,0) );
	self.searchP.SearchLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	self.searchP.SearchLabel:SetForeColor( Turbine.UI.Color(1,.9,.5) );
	self.searchP.SearchLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.searchP.SearchLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight );
	self.searchP.SearchLabel:SetText( L'search'..":" );

	self.searchP.SearchInput = Turbine.UI.Lotro.TextBox(); --text box for search term
	self.searchP.SearchInput:SetParent( self.searchP );
	self.searchP.SearchInput.PosOffsetX = 8;
	self.searchP.SearchInput:SetPosition( self.searchP.SearchLabel:GetLeft() + self.searchP.SearchLabel:GetWidth() + self.searchP.SearchInput.PosOffsetX, 6 );
	self.searchP.SearchInput:SetSize( 105, 20 );
	self.searchP.SearchInput:SetMultiline( false );
	self.searchP.SearchInput:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 );
	self.searchP.SearchInput:SetForeColor( Turbine.UI.Color(1,.9,.5) );
	self.searchP.SearchInput.Text = "";
	self.searchP.SearchInput:SetText( self.searchP.SearchInput.Text );
	self.searchP.SearchInput:SetWantsKeyEvents( true );
	self.searchP.SearchInput:SetWantsUpdates( false );

	self.searchP.SearchInput.KeyDown = function( sender, args )
		if ( args.Action == 19 ) then -- right mouse button click
			self.searchP.SearchInput:SetWantsUpdates( false );
			self.searchP.SearchInput:SetText( "" );
		end
		self:Search();
	end

	self.searchP.SearchInput.FocusGained = function()
		self.searchP.SearchInput:SetWantsUpdates( true );
	end
	
	self.searchP.SearchInput.FocusLost = function()
		self.searchP.SearchInput:SetWantsUpdates( false );
	end
	
	self.searchP.SearchInput.Update = function()
		if self.searchP.SearchInput.Text ~= self.searchP.SearchInput:GetText() then
			self.searchP.SearchInput.Text = self.searchP.SearchInput:GetText()
			self:Search();
		end
	end

	self.searchP.ModeCheckBox = Turbine.UI.Lotro.CheckBox();
	self.searchP.ModeCheckBox:SetParent( self.searchP );
	self.searchP.ModeCheckBox.PosOffsetX = 2;
	self.searchP.ModeCheckBox:SetPosition( self.searchP:GetWidth() - self.searchP.ModeCheckBox:GetWidth() - self.searchP.ModeCheckBox.PosOffsetX, -9 );
	self.searchP.ModeCheckBox:SetChecked( self.Settings.BPW_MiniMode );
	self.searchP.ModeCheckBox:SetOutlineColor( Turbine.UI.Color(0,0,0) );
	self.searchP.ModeCheckBox:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	self.searchP.ModeCheckBox:SetForeColor( Turbine.UI.Color(1,.9,.5) );
	self.searchP.ModeCheckBox:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.searchP.ModeCheckBox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.searchP.ModeCheckBox:SetText( L'miniMode' );

	self.searchP.ModeCheckBox.CheckedChanged = function( sender, args )
		self:SetMode( self.searchP.ModeCheckBox:IsChecked() );
	end

	self.searchP.SizeChanged = function( sender, args )
		local width, height = self.searchP:GetSize();
		self.searchP.ModeCheckBox:SetLeft( width - self.searchP.ModeCheckBox:GetWidth() - self.searchP.ModeCheckBox.PosOffsetX );
	end

	-- itemP(anel) and itemListBox

	-- Initialize the container that holds the elements referencing
	-- the player's inventory.
	self.items = { };

	self.itemP = Turbine.UI.Control();
	self.itemP:SetParent( self );
	self.itemP.PosOffsetX = 9;
	self.itemP.PosOffsetY = 2;
	self.itemP:SetSize( self:GetWidth() - self.itemP.PosOffsetX * 2, self:GetHeight() - ( self.searchP:GetTop() + self.searchP:GetHeight() + self.itemP.PosOffsetY * 2 + self.bottonP.SizeY ) );
	self.itemP:SetPosition( self.itemP.PosOffsetX, self.searchP:GetTop() + self.searchP:GetHeight() + self.itemP.PosOffsetY );

	self.itemP.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemP.itemListBoxScrollBar.PosOffsetX = 1;
	self.itemP.itemListBoxScrollBar.PosOffsetY = 1;
	self.itemP.itemListBoxScrollBar.width = 10;
	self.itemP.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemP.itemListBoxScrollBar:SetParent( self.itemP );
	--self.itemP.itemListBoxScrollBar:SetBackColor(Turbine.UI.Color(0,0,0));
	self.itemP.itemListBoxScrollBar:SetPosition( self.itemP:GetWidth() - self.itemP.itemListBoxScrollBar.width - self.itemP.itemListBoxScrollBar.PosOffsetX, self.itemP.itemListBoxScrollBar.PosOffsetY );
	self.itemP.itemListBoxScrollBar:SetSize( self.itemP.itemListBoxScrollBar.width, self.itemP:GetHeight() - self.itemP.itemListBoxScrollBar.PosOffsetY * 2 );

	self.itemP.itemListBoxOuterF = Turbine.UI.Control();
	self.itemP.itemListBoxOuterF:SetParent(self.itemP);
	self.itemP.itemListBoxOuterF.PosOffsetX = 1;
	self.itemP.itemListBoxOuterF.PosOffsetY = 1;
	self.itemP.itemListBoxOuterF:SetSize( self.itemP:GetWidth() - self.itemP.itemListBoxOuterF.PosOffsetX * 2 - self.itemP.itemListBoxScrollBar.PosOffsetX * 2 - self.itemP.itemListBoxScrollBar.width, self.itemP:GetHeight() - self.itemP.itemListBoxOuterF.PosOffsetY * 2 );
	self.itemP.itemListBoxOuterF:SetPosition( self.itemP.itemListBoxOuterF.PosOffsetY, self.itemP.itemListBoxOuterF.PosOffsetY );
	self.itemP.itemListBoxOuterF:SetBackColor( Turbine.UI.Color(.1,.1,.2) );

	self.itemP.itemListBoxInnerF = Turbine.UI.Control();
	self.itemP.itemListBoxInnerF:SetParent( self.itemP.itemListBoxOuterF );
	self.itemP.itemListBoxInnerF.PosOffsetX = 1;
	self.itemP.itemListBoxInnerF.PosOffsetY = 1;
	self.itemP.itemListBoxInnerF:SetSize( self.itemP.itemListBoxOuterF:GetWidth() - self.itemP.itemListBoxInnerF.PosOffsetX * 2, self.itemP.itemListBoxOuterF:GetHeight() - self.itemP.itemListBoxInnerF.PosOffsetY * 2 );
	self.itemP.itemListBoxInnerF:SetPosition( self.itemP.itemListBoxInnerF.PosOffsetY, self.itemP.itemListBoxInnerF.PosOffsetY );
	self.itemP.itemListBoxInnerF:SetBackColor( Turbine.UI.Color(0,0,0) );

	self.itemP.itemListBox = Turbine.UI.ListBox();
	self.itemP.itemListBox:SetParent( self.itemP.itemListBoxInnerF );
	self.itemP.itemListBox:SetVerticalScrollBar( self.itemP.itemListBoxScrollBar );
	self.itemP.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemP.itemListBox:SetAllowDrop( true );
	self.itemP.itemListBox:SetBackColor( Turbine.UI.Color(0,0,0) );
	self.itemP.itemListBox.PosOffsetX = 4;
	self.itemP.itemListBox.PosOffsetY = 4;
	self.itemP.itemListBox:SetLeft( self.itemP.itemListBox.PosOffsetX );
	self.itemP.itemListBox:SetWidth( self.itemP.itemListBoxInnerF:GetWidth() - self.itemP.itemListBox.PosOffsetX * 2 );

	self.itemP.itemListBox:SetTop( self.itemP.itemListBox.PosOffsetY );
	self.itemP.itemListBox:SetHeight( self.itemP.itemListBoxInnerF:GetHeight() - self.itemP.itemListBox.PosOffsetY * 2 );
	
	self.itemP.itemListBox.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();
		if ( shortcut ~= nil ) then
			local destinationItemControl = self.itemP.itemListBox:GetItemAt( args.X, args.Y );
			local destinationIndex = self.itemP.itemListBox:IndexOfItem( destinationItemControl );
			if ( destinationIndex > 0 and destinationIndex <= self.backpackSize ) then
				self.backpack:PerformShortcutDrop( shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
			end
		end
	end

	self.itemP.itemListBox.SetMaxItemsPerLine = function( )
		local function calcMaxItemsPerLine( width )
			local itemWidth = 40;

			if ( self.itemP.itemListBox:GetItem(1) ~= nil ) then
				itemWidth = self.itemP.itemListBox:GetItem(1):GetWidth();
			end

			return width / itemWidth;
		end
		Turbine.UI.ListBox.SetMaxItemsPerLine( self.itemP.itemListBox, calcMaxItemsPerLine( self.itemP.itemListBox:GetWidth() ) );
	end

	self.itemP.itemListBox.SizeChanged = function( sender, args )
		self.itemP.itemListBox:SetMaxItemsPerLine();
	end

	self.itemP.itemListBoxInnerF.SizeChanged = function( sender, args )
		local width, height = self.itemP.itemListBoxInnerF:GetSize();

		local listWidth = width - self.itemP.itemListBox.PosOffsetX * 2;
		local listHeight = height - self.itemP.itemListBox.PosOffsetY * 2;

		self.itemP.itemListBox:SetSize( listWidth, listHeight );
	end

	self.itemP.itemListBoxOuterF.SizeChanged = function( sender, args )
		local width, height = self.itemP.itemListBoxOuterF:GetSize();

		self.itemP.itemListBoxInnerF:SetSize( width - self.itemP.itemListBoxInnerF.PosOffsetX * 2, height - self.itemP.itemListBoxInnerF.PosOffsetY * 2 );
	end

	self.itemP.SizeChanged = function( sender, args )
		local width, height = self.itemP:GetSize();

		self.itemP.itemListBoxScrollBar:SetHeight( height - self.itemP.itemListBoxScrollBar.PosOffsetY * 2 );
		self.itemP.itemListBoxScrollBar:SetLeft( width - self.itemP.itemListBoxScrollBar.width - self.itemP.itemListBoxScrollBar.PosOffsetX );
		self.itemP.itemListBoxOuterF:SetSize( width - self.itemP.itemListBoxOuterF.PosOffsetX * 2 - self.itemP.itemListBoxScrollBar.PosOffsetX * 2 - self.itemP.itemListBoxScrollBar.width, height - self.itemP.itemListBoxOuterF.PosOffsetY * 2 );
	end

	-- resizeH and its controls
	self.resizeH = Turbine.UI.Control();
	self.resizeH:SetParent( self );
	self.resizeH:SetZOrder( 100 );
	self.resizeH:SetSize( 20, 20 );
	self.resizeH:SetPosition( self:GetWidth() - self.resizeH:GetWidth(), self:GetHeight() - self.resizeH:GetHeight() );

	self.resizeH.MouseDown = function( sender, args )
		sender.dragStartX = args.X;
		sender.dragStartY = args.Y;
		sender.dragging = true;
	end

	self.resizeH.MouseMove = function( sender, args )
		local width, height = self:GetSize();

		if ( sender.dragging ) then
			local minheight = self.topP.PosOffsetY + 0 + self.searchP.SizeY + self.itemP.PosOffsetY + (44 + 4) * 3 + self.bottonP.PosOffsetY + self.bottonP.SizeY;
			if not self.Settings.BPW_MiniMode then minheight = minheight + self.topP.SizeY + self.searchP.PosOffsetY end
			
			local minwidth = self.itemP.PosOffsetX + self.itemP.itemListBoxScrollBar.PosOffsetX + self.itemP.itemListBoxScrollBar.width + self.itemP.itemListBoxOuterF.PosOffsetX * 2 + self.itemP.itemListBoxInnerF.PosOffsetX * 2 + self.itemP.itemListBox.PosOffsetX * 2 + (40 + 2) * 5;
			if not self.Settings.BPW_MiniMode then minwidth = self.topP.capP:GetWidth() + self.topP.PosOffsetX * 4 + 1; end --402
			
			local newwidth = math.max( width + ( args.X - sender.dragStartX ), minwidth ); -- beware of minimum width
			local newheight = math.max( height + ( args.Y - sender.dragStartY ), minheight ); -- beware of minimum height

			self:SetSize( newwidth, newheight );
			sender:SetPosition( newwidth - sender:GetWidth(), newheight - sender:GetHeight() );
			self:PerformLayout();
		end
	end

	self.resizeH.MouseUp = function( sender, args )
		sender.dragging = false;
	end

	-- backpack
	local player = Turbine.Gameplay.LocalPlayer.GetInstance();
	self.backpack = player:GetBackpack();

	self.backpackSize = self.backpack:GetSize();
	self.backpackFilled = 0;

	self.backpack.ItemAdded = function( sender, args )
		--Turbine.Shell.WriteLine( "ItemAdded: " .. tostring(args.Index) );
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		self:RefreshItems();
	end

	self.backpack.ItemRemoved = function( sender, args )
		--Turbine.Shell.WriteLine( "ItemRemoved: " .. tostring(args.Index) );
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		self:RefreshItems();
	end

	self.backpack.ItemMoved = function( sender, args )
		--Turbine.Shell.WriteLine( "ItemMoved: " .. tostring(args.OldIndex) .. " - " .. tonumber(args.NewIndex) );
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
		self:RefreshItems();
	end

	self.backpack.SizeChanged = function( sender, args )
		self:RefreshItems();
	end

	self:SetMode( self.Settings.BPW_MiniMode, true );
	self:RefreshItems();
	self:PerformLayout();
end


--[[
	-- BigPackWindow Getter / Setter
--]]

function BigPackWindow:SetOpacity( value )
	if ( value ~= nil and 0 < value and value <= 1 ) then
		Turbine.UI.Lotro.Window.SetOpacity( self, value );
		self.Settings.BPW_Opacity = value;
		self:Save();
	end
end

function BigPackWindow:SetTitle( value )
	if ( type(value) == 'string' ) then
		self.Settings.BPW_WindowTitle = value;
		self:Save();
		self:RefreshCapacity();
	end
end

function BigPackWindow:GetTitle()
	return self.Settings.BPW_WindowTitle;
end

function BigPackWindow:SetMode( mode, onLoad )
	if type( mode ) == 'boolean' then
		self.Settings.BPW_MiniMode = mode;
		self:Save();
	end
	if ( self.searchP.ModeCheckBox:IsChecked() ~= self.Settings.BPW_MiniMode ) then
		self.searchP.ModeCheckBox:SetChecked( self.Settings.BPW_MiniMode );
		return;
	end

	if self.Settings.BPW_MiniMode then
		if not onLoad then
			self:SetHeight( self:GetHeight() - self.topP.SizeY - self.searchP.PosOffsetY );
		end
		self.topP:SetVisible( false );
		self.topP:SetHeight( 0 );
		self.searchP:SetTop( self.topP.PosOffsetY );
	else
		if not onLoad then
			self:SetHeight( self:GetHeight() + self.topP.SizeY + self.searchP.PosOffsetY );
		end
		self.topP:SetVisible( true );
		self.topP:SetHeight( self.topP.SizeY );
		self.searchP:SetTop( self.topP.PosOffsetY + self.topP.SizeY + self.searchP.PosOffsetY );
	end

	self.itemP:SetTop( self.searchP:GetTop() + self.searchP:GetHeight() + self.itemP.PosOffsetY );
	self.resizeH:SetTop( self:GetHeight() - self.resizeH:GetHeight() );

	self:RefreshCapacity();
end

function BigPackWindow:GetMode()
	return self.Settings.BPW_MiniMode;
end


--[[
	-- BigPackWindow Paint UI
--]]

function BigPackWindow:RefreshCapacity()
	if self.Settings.BPW_MiniMode then
		self:SetText( self.Settings.BPW_WindowTitle .. " (" .. self.backpackFilled .. "/" .. self.backpackSize .. ")" );
	else
		self:SetText( self.Settings.BPW_WindowTitle );
	end

	self.topP.capP.Display:SetText( L'capacity' .. " (" .. self.backpackFilled .. "/" .. self.backpackSize .. ")" );
	self.topP.capP.Filler:SetSize( self.topP.capP.Filler.MaxWidth * ( self.backpackFilled / self.backpackSize ), 17 );
	self.topP.capP.Filler:SetBackColor( Turbine.UI.Color(.65,0,.5,0) );
	if ( self.backpackFilled / self.backpackSize ) >= .75 then
		self.topP.capP.Filler:SetBackColor( Turbine.UI.Color(.65,.5,0,0) );
	elseif ( self.backpackFilled / self.backpackSize ) >= .5 then
		self.topP.capP.Filler:SetBackColor( Turbine.UI.Color(.65,1,1,0) );
	end
end

function BigPackWindow:RefreshItems()
	local tmpItem;
	local tmpFilled = 0;

	self.itemP.itemListBox:ClearItems();

	for i = 1, self.backpackSize, 1 do
		tmpItemP = Turbine.UI.Control();
		tmpItemP.itemIndex = i;
		tmpItemP.Frame = Turbine.UI.Control()
		tmpItemP.Frame:SetParent( tmpItemP )
		tmpItemP.Frame:SetPosition( 1, 0 );
		tmpItemP.Frame.ItemL = Turbine.UI.Label();
		tmpItemP.Frame.ItemL:SetParent( tmpItemP.Frame )

		tmpItem = self.backpack:GetItem( i );
		self.items[i] = Turbine.UI.Lotro.ItemControl( tmpItem );
		self.items[i]:SetParent( tmpItemP.Frame );
		self.items[i]:SetPosition( 1, 2 );

		tmpItemP:SetSize( 40, 44 );
		tmpItemP.Frame:SetSize( 40, 42 );
		tmpItemP.Frame:SetBackground( defaultResources.Item.Background );

		self.itemP.itemListBox:AddItem( tmpItemP );

		if ( tmpItem ~= nil ) then
			tmpFilled = tmpFilled + 1;
		end
	end
	
	self.backpackFilled = tmpFilled;

	self.itemP.itemListBox:SetMaxItemsPerLine();

	self:RefreshCapacity();
	--self:PerformLayout();
	--self:Search();
end

function BigPackWindow:Search()
	local searchString = "";
	local tmpItem;

	if self.searchP.SearchInput then
		searchString = string.upper( self.searchP.SearchInput:GetText() );
	end

	for i = 1, self.itemP.itemListBox:GetItemCount(), 1 do
		tmpItemP = self.itemP.itemListBox:GetItem( i );
		tmpBackPackItem = self.backpack:GetItem( tmpItemP.itemIndex );
		if tmpItemP and tmpBackPackItem then
			if string.find( string.upper( tmpBackPackItem:GetName() ), searchString ) or ( searchString == "" ) then
				self.items[tmpItemP.itemIndex]:SetVisible( true );
			else
				self.items[tmpItemP.itemIndex]:SetVisible( false );
			end
		end
	end
end

function BigPackWindow:PerformLayout()
	self:Layout( { } )
end

function BigPackWindow:Layout( args )
	local width, height = self:GetSize();

	self.topP:SetWidth( width - self.topP.PosOffsetX * 2 );
	self.searchP:SetWidth( width - self.searchP.PosOffsetX * 2 );

	self.itemP:SetSize( width - self.itemP.PosOffsetX * 2, height - ( self.searchP:GetTop() + self.searchP:GetHeight() + self.itemP.PosOffsetY * 2 + self.bottonP.SizeY ) );
	self.itemP:SetPosition( self.itemP.PosOffsetX, self.searchP:GetTop() + self.searchP:GetHeight() + self.itemP.PosOffsetY );
	--self.itemP.itemListBox:SetTop( self.itemP.itemListBox.PosOffsetY );
	--self.itemP.itemListBox:SetHeight( self.itemP.itemListBoxInnerF:GetHeight() - self.itemP.itemListBox.PosOffsetY * 2 );
end


--[[
	-- BigPackWindow Load / Save
--]]

function BigPackWindow:SetDefaults()
	self.Defaults.BPW_SettingsVersion	= 1;
	self.Defaults.BPW_WindowTitle		= L'windowTitle';
	self.Defaults.BPW_Width				= 402; --self.topP.capP:GetWidth() + self.topP.PosOffsetX * 4 + 1
	self.Defaults.BPW_Height			= 438; --self.topP.PosOffsetY + self.topP.SizeY + self.searchP.PosOffsetY + self.searchP.SizeY + self.itemP.PosOffsetY + 44 * 6 + self.bottonP.PosOffsetY + self.bottonP.SizeY
	self.Defaults.BPW_Opacity			= 0.8; -- valid values: 0.0 - 1.0
	self.Defaults.BPW_Visible			= true;
	self.Defaults.BPW_MiniMode			= false;
	self.Defaults.BPW_Left				= ( Turbine.UI.Display.GetWidth() - self.Defaults.BPW_Width ) / 2;
	self.Defaults.BPW_Top				= ( Turbine.UI.Display.GetHeight() - self.Defaults.BPW_Height ) / 2;
end

function BigPackWindow:Load()
	local fLoad = Turbine.PluginData.Load; -- VindarPatch may not touch me
	local tSettings = fLoad( Turbine.DataScope.Character, "BigPackSettings" );

	if ( type(tSettings) ~= "table" ) then
		-- set defaults
		self.Settings = PTweety.BigPack.Utils.DeepCopy( self.Defaults );
	else
		if ( Turbine.Engine.GetLocale() ~= "en" ) then
			-- decode strings to numbers
			tSettings = PTweety.BigPack.Utils.DeepCopyDecode( tSettings );
		end
		self.Settings = tSettings;
	end
end

function BigPackWindow:Save()
	local fSave = Turbine.PluginData.Save; -- VindarPatch may not touch me
	local tSettings = {};

	if ( Turbine.Engine.GetLocale() ~= "en" ) then
		-- encode numbers to strings
		tSettings = PTweety.BigPack.Utils.DeepCopyEncode( self.Settings );
	else
		tSettings = PTweety.BigPack.Utils.DeepCopy( self.Settings );
	end

	fSave( Turbine.DataScope.Character, "BigPackSettings", tSettings );
end
